@@ -72,7 +72,7 @@ class ChecksumTestCase(unittest.TestCase):
         self.assertEqual(binascii.crc32(b'spam'), zlib.crc32(b'spam'))
 
 
-# Issue #10276 - check that inputs >=4GB are handled correctly.
+# Issue #10276 - check that inputs >=4 GiB are handled correctly.
 class ChecksumBigBufferTestCase(unittest.TestCase):
 
     @bigmemtest(size=_4G + 4, memuse=1, dry_run=False)
@@ -130,7 +130,7 @@ class ExceptionTestCase(unittest.TestCase):
 class BaseCompressTestCase(object):
     def check_big_compress_buffer(self, size, compress_func):
         _1M = 1024 * 1024
-        # Generate 10MB worth of random, and expand it by repeating it.
+        # Generate 10 MiB worth of random, and expand it by repeating it.
         # The assumption is that zlib's memory is not big enough to exploit
         # such spread out redundancy.
         data = b''.join([random.getrandbits(8 * _1M).to_bytes(_1M, 'little')
@@ -164,6 +164,16 @@ class CompressTestCase(BaseCompressTestCase, unittest.TestCase):
         x = zlib.compress(HAMLET_SCENE)
         self.assertEqual(zlib.decompress(x), HAMLET_SCENE)
 
+    def test_keywords(self):
+        x = zlib.compress(HAMLET_SCENE, level=3)
+        self.assertEqual(zlib.decompress(x), HAMLET_SCENE)
+        with self.assertRaises(TypeError):
+            zlib.compress(data=HAMLET_SCENE, level=3)
+        self.assertEqual(zlib.decompress(x,
+                                         wbits=zlib.MAX_WBITS,
+                                         bufsize=zlib.DEF_BUF_SIZE),
+                         HAMLET_SCENE)
+
     def test_speech128(self):
         # compress more data
         data = HAMLET_SCENE * 128
@@ -234,6 +244,27 @@ class CompressObjectTestCase(BaseCompressTestCase, unittest.TestCase):
             self.assertIsInstance(dco.unconsumed_tail, bytes)
             self.assertIsInstance(dco.unused_data, bytes)
 
+    def test_keywords(self):
+        level = 2
+        method = zlib.DEFLATED
+        wbits = -12
+        memLevel = 9
+        strategy = zlib.Z_FILTERED
+        co = zlib.compressobj(level=level,
+                              method=method,
+                              wbits=wbits,
+                              memLevel=memLevel,
+                              strategy=strategy,
+                              zdict=b"")
+        do = zlib.decompressobj(wbits=wbits, zdict=b"")
+        with self.assertRaises(TypeError):
+            co.compress(data=HAMLET_SCENE)
+        with self.assertRaises(TypeError):
+            do.decompress(data=zlib.compress(HAMLET_SCENE))
+        x = co.compress(HAMLET_SCENE) + co.flush()
+        y = do.decompress(x, max_length=len(HAMLET_SCENE)) + do.flush()
+        self.assertEqual(HAMLET_SCENE, y)
+
     def test_compressoptions(self):
         # specify lots of options to compressobj()
         level = 2
@@ -249,10 +280,6 @@ class CompressObjectTestCase(BaseCompressTestCase, unittest.TestCase):
         y2 = dco.flush()
         self.assertEqual(HAMLET_SCENE, y1 + y2)
 
-        # keyword arguments should also be supported
-        zlib.compressobj(level=level, method=method, wbits=wbits,
-            memLevel=memLevel, strategy=strategy, zdict=b"")
-
     def test_compressincremental(self):
         # compress object in steps, decompress object as one-shot
         data = HAMLET_SCENE * 128
@@ -407,7 +434,8 @@ class CompressObjectTestCase(BaseCompressTestCase, unittest.TestCase):
     def test_flushes(self):
         # Test flush() with the various options, using all the
         # different levels in order to provide more variations.
-        sync_opt = ['Z_NO_FLUSH', 'Z_SYNC_FLUSH', 'Z_FULL_FLUSH']
+        sync_opt = ['Z_NO_FLUSH', 'Z_SYNC_FLUSH', 'Z_FULL_FLUSH',
+                    'Z_PARTIAL_FLUSH', 'Z_BLOCK']
         sync_opt = [getattr(zlib, opt) for opt in sync_opt
                     if hasattr(zlib, opt)]
         data = HAMLET_SCENE * 8
@@ -724,10 +752,15 @@ class CompressObjectTestCase(BaseCompressTestCase, unittest.TestCase):
     def test_wbits(self):
         # wbits=0 only supported since zlib v1.2.3.5
         # Register "1.2.3" as "1.2.3.0"
-        v = (zlib.ZLIB_RUNTIME_VERSION + ".0").split(".", 4)
-        supports_wbits_0 = int(v[0]) > 1 or int(v[0]) == 1 \
-            and (int(v[1]) > 2 or int(v[1]) == 2
-            and (int(v[2]) > 3 or int(v[2]) == 3 and int(v[3]) >= 5))
+        # or "1.2.0-linux","1.2.0.f","1.2.0.f-linux"
+        v = zlib.ZLIB_RUNTIME_VERSION.split('-', 1)[0].split('.')
+        if len(v) < 4:
+            v.append('0')
+        elif not v[-1].isnumeric():
+            v[-1] = '0'
+
+        v = tuple(map(int, v))
+        supports_wbits_0 = v >= (1, 2, 3, 5)
 
         co = zlib.compressobj(level=1, wbits=15)
         zlib15 = co.compress(HAMLET_SCENE) + co.flush()
