@@ -2,19 +2,21 @@
 """
 This script is used to build "official" universal installers on macOS.
 
-NEW for 3.6.8 / 2.7.16:
-- also build and use Tk 8.6 for 10.6+ installers
-NEW for 3.6.5:
+NEW for 3.7.0:
 - support Intel 64-bit-only () and 32-bit-only installer builds
-- build and link with private Tcl/Tk 8.6 for 10.9+ builds
+- build and use internal Tcl/Tk 8.6 for 10.6+ builds
 - deprecate use of explicit SDK (--sdk-path=) since all but the oldest
   versions of Xcode support implicit setting of an SDK via environment
   variables (SDKROOT and friends, see the xcrun man page for more info).
   The SDK stuff was primarily needed for building universal installers
-  for 10.4; so as of 3.6.5, building installers for 10.4 is no longer
+  for 10.4; so as of 3.7.0, building installers for 10.4 is no longer
   supported with build-installer.
 - use generic "gcc" as compiler (CC env var) rather than "gcc-4.2"
 
+TODO:
+- support SDKROOT and DEVELOPER_DIR xcrun env variables
+- test with 10.5 and 10.4 and determine support status
+
 Please ensure that this script keeps working with Python 2.5, to avoid
 bootstrap issues (/usr/bin/python is Python 2.5 on OSX 10.5).  Doc builds
 use current versions of Sphinx and require a reasonably current python3.
@@ -22,15 +24,16 @@ Sphinx and dependencies are installed into a venv using the python3's pip
 so will fetch them from PyPI if necessary.  Since python3 is now used for
 Sphinx, build-installer.py should also be converted to use python3!
 
-For 10.6 or greater deployment targets, build-installer builds and links
-with its own copy of Tcl/Tk 8.6 and the rest of this paragraph does not
-apply.  Otherwise, build-installer requires an installed third-party version
-of Tcl/Tk 8.4 (for OS X 10.4 and 10.5 deployment targets) or Tcl/TK 8.5
-(for 10.6 or later) installed in /Library/Frameworks.  When installed,
+For 3.7.0, when building for a 10.6 or higher deployment target,
+build-installer builds and links with its own copy of Tcl/Tk 8.6.
+Otherwise, it requires an installed third-party version of
+Tcl/Tk 8.4 (for OS X 10.4 and 10.5 deployment targets), Tcl/TK 8.5
+(for 10.6 or later), or Tcl/TK 8.6 (for 10.9 or later)
+installed in /Library/Frameworks.  When installed,
 the Python built by this script will attempt to dynamically link first to
 Tcl and Tk frameworks in /Library/Frameworks if available otherwise fall
 back to the ones in /System/Library/Framework.  For the build, we recommend
-installing the most recent ActiveTcl 8.5 or 8.4 version, depending
+installing the most recent ActiveTcl 8.6. 8.5, or 8.4 version, depending
 on the deployment target.  The actual version linked to depends on the
 path of /Library/Frameworks/{Tcl,Tk}.framework/Versions/Current.
 
@@ -164,8 +167,7 @@ def getTargetCompilers():
 
 CC, CXX = getTargetCompilers()
 
-PYTHON_2 = getVersionMajorMinor()[0] == 2
-PYTHON_3 = getVersionMajorMinor()[0] == 3
+PYTHON_3 = getVersionMajorMinor() >= (3, 0)
 
 USAGE = textwrap.dedent("""\
     Usage: build_python [options]
@@ -213,9 +215,9 @@ def library_recipes():
 
     result.extend([
           dict(
-              name="OpenSSL 1.0.2q",
-              url="https://www.openssl.org/source/openssl-1.0.2q.tar.gz",
-              checksum='7563e1ce046cb21948eeb6ba1a0eb71c',
+              name="OpenSSL 1.1.0h",
+              url="https://www.openssl.org/source/openssl-1.1.0h.tar.gz",
+              checksum='5271477e4d93f4ea032b665ef095ff24',
               buildrecipe=build_universal_openssl,
               configure=None,
               install=None,
@@ -554,7 +556,7 @@ def checkEnvironment():
     # Tcl/Tk, if we are not using building and using our own private copy of
     # Tcl/Tk, ensure:
     # 1. there is a user-installed framework (usually ActiveTcl) in (or linked
-    #       in) SDKROOT/Library/Frameworks.  As of Python 3.6.5, we no longer
+    #       in) SDKROOT/Library/Frameworks.  As of Python 3.7.0, we no longer
     #       enforce that the version of the user-installed framework also
     #       exists in the system-supplied Tcl/Tk frameworks.  Time to support
     #       Tcl/Tk 8.6 even if Apple does not.
@@ -610,10 +612,9 @@ def checkEnvironment():
         base_path = base_path + ':' + OLD_DEVELOPER_TOOLS
     os.environ['PATH'] = base_path
     print("Setting default PATH: %s"%(os.environ['PATH']))
-    if PYTHON_2:
-        # Ensure we have access to sphinx-build.
-        # You may have to define SDK_TOOLS_BIN and link to it there,
-        runCommand('sphinx-build --version')
+    # Ensure we have access to sphinx-build.
+    # You may have to create a link in /usr/bin for it.
+    runCommand('sphinx-build --version')
 
 def parseOptions(args=None):
     """
@@ -810,34 +811,23 @@ def build_universal_openssl(basedir, archList):
             "ppc64": ["darwin64-ppc-cc"],
         }
         configure_opts = [
-            "no-krb5",
             "no-idea",
             "no-mdc2",
             "no-rc5",
             "no-zlib",
-            "enable-tlsext",
-            "no-ssl2",
             "no-ssl3",
             # "enable-unit-test",
             "shared",
-            "--install_prefix=%s"%shellQuote(archbase),
             "--prefix=%s"%os.path.join("/", *FW_VERSION_PREFIX),
             "--openssldir=%s"%os.path.join("/", *FW_SSL_DIRECTORY),
         ]
         if no_asm:
             configure_opts.append("no-asm")
-        # OpenSSL 1.0.2o broke the Configure test for whether the compiler
-        # in use supports dependency rule generation (cc -M) with gcc-4.2
-        # used for the 10.6+ installer builds.  Patch Configure here to
-        # force use of "cc -M" rather than "makedepend".
-        runCommand(
-            """sed -i "" 's|my $cc_as_makedepend = 0|my $cc_as_makedepend = 1|g' Configure""")
-
         runCommand(" ".join(["perl", "Configure"]
                         + arch_opts[arch] + configure_opts))
         runCommand("make depend")
         runCommand("make all")
-        runCommand("make install_sw")
+        runCommand("make install_sw DESTDIR=%s"%shellQuote(archbase))
         # runCommand("make test")
         return
 
@@ -1066,14 +1056,9 @@ def buildPythonDocs():
     curDir = os.getcwd()
     os.chdir(buildDir)
     runCommand('make clean')
-    if PYTHON_2:
-        # Python 2 doc builds do not use blurb nor do they have a venv target.
-        # Assume sphinx-build is on our PATH, checked in checkEnvironment
-        runCommand('make html')
-    else:
-        # Create virtual environment for docs builds with blurb and sphinx
-        runCommand('make venv')
-        runCommand('make html PYTHON=venv/bin/python')
+    # Create virtual environment for docs builds with blurb and sphinx
+    runCommand('make venv')
+    runCommand('make html PYTHON=venv/bin/python')
     os.chdir(curDir)
     if not os.path.exists(docdir):
         os.mkdir(docdir)
@@ -1533,27 +1518,16 @@ def buildDMG():
     imagepath = imagepath + '.dmg'
 
     os.mkdir(outdir)
-
-    # Try to mitigate race condition in certain versions of macOS, e.g. 10.9,
-    # when hdiutil create fails with  "Resource busy".  For now, just retry
-    # the create a few times and hope that it eventually works.
-
     volname='Python %s'%(getFullVersion())
-    cmd = ("hdiutil create -format UDRW -volname %s -srcfolder %s -size 100m %s"%(
+    runCommand("hdiutil create -format UDRW -volname %s -srcfolder %s %s"%(
             shellQuote(volname),
             shellQuote(os.path.join(WORKDIR, 'installer')),
             shellQuote(imagepath + ".tmp.dmg" )))
-    for i in range(5):
-        fd = os.popen(cmd, 'r')
-        data = fd.read()
-        xit = fd.close()
-        if not xit:
-            break
-        sys.stdout.write(data)
-        print(" -- retrying hdiutil create")
-        time.sleep(5)
-    else:
-        raise RuntimeError("command failed: %s"%(commandline,))
+
+    # Try to mitigate race condition in certain versions of macOS, e.g. 10.9,
+    # when hdiutil fails with  "Resource busy"
+
+    time.sleep(10)
 
     if not os.path.exists(os.path.join(WORKDIR, "mnt")):
         os.mkdir(os.path.join(WORKDIR, "mnt"))
