@@ -10,7 +10,7 @@ import tempfile
 from itertools import chain
 from pathlib import Path
 from zipfile import ZipFile, ZIP_DEFLATED
-import subprocess
+
 
 TKTCL_RE = re.compile(r'^(_?tk|tcl).+\.(pyd|dll)', re.IGNORECASE)
 DEBUG_RE = re.compile(r'_d\.(pyd|dll|exe|pdb|lib)$', re.IGNORECASE)
@@ -20,6 +20,7 @@ DEBUG_FILES = {
     '_ctypes_test',
     '_testbuffer',
     '_testcapi',
+    '_testconsole',
     '_testimportmultiple',
     '_testmultiphase',
     'xxlimited',
@@ -45,11 +46,14 @@ EXCLUDE_FILE_FROM_LIBRARY = {
 }
 
 EXCLUDE_FILE_FROM_LIBS = {
-    'ssleay',
-    'libeay',
+    'liblzma',
     'python3stub',
 }
 
+EXCLUDED_FILES = {
+    'pyshellext',
+}
+
 def is_not_debug(p):
     if DEBUG_RE.search(p.name):
         return False
@@ -57,7 +61,7 @@ def is_not_debug(p):
     if TKTCL_RE.search(p.name):
         return False
 
-    return p.stem.lower() not in DEBUG_FILES
+    return p.stem.lower() not in DEBUG_FILES and p.stem.lower() not in EXCLUDED_FILES
 
 def is_not_debug_or_python(p):
     return is_not_debug(p) and not PYTHON_DLL_RE.search(p.name)
@@ -67,8 +71,6 @@ def include_in_lib(p):
     if p.is_dir():
         if name in EXCLUDE_FROM_LIBRARY:
             return False
-        if name.startswith('plat-'):
-            return False
         if name == 'test' and p.parts[-2].lower() == 'lib':
             return False
         if name in {'test', 'tests'} and p.parts[-3].lower() == 'lib':
@@ -99,11 +101,13 @@ def include_in_tools(p):
 
     return p.suffix.lower() in {'.py', '.pyw', '.txt'}
 
+BASE_NAME = 'python{0.major}{0.minor}'.format(sys.version_info)
+
 FULL_LAYOUT = [
     ('/', '$build', 'python.exe', is_not_debug),
     ('/', '$build', 'pythonw.exe', is_not_debug),
-    ('/', '$build', 'python{0.major}.dll'.format(sys.version_info), is_not_debug),
-    ('/', '$build', 'python{0.major}{0.minor}.dll'.format(sys.version_info), is_not_debug),
+    ('/', '$build', 'python{}.dll'.format(sys.version_info.major), is_not_debug),
+    ('/', '$build', '{}.dll'.format(BASE_NAME), is_not_debug),
     ('DLLs/', '$build', '*.pyd', is_not_debug),
     ('DLLs/', '$build', '*.dll', is_not_debug_or_python),
     ('include/', 'include', '*.h', None),
@@ -117,7 +121,7 @@ EMBED_LAYOUT = [
     ('/', '$build', 'python*.exe', is_not_debug),
     ('/', '$build', '*.pyd', is_not_debug),
     ('/', '$build', '*.dll', is_not_debug),
-    ('python{0.major}{0.minor}.zip'.format(sys.version_info), 'Lib', '**/*', include_in_embeddable_lib),
+    ('{}.zip'.format(BASE_NAME), 'Lib', '**/*', include_in_embeddable_lib),
 ]
 
 if os.getenv('DOC_FILENAME'):
@@ -181,12 +185,12 @@ def main():
     parser.add_argument('-o', '--out', metavar='file', help='The name of the output archive', type=Path, default=None)
     parser.add_argument('-t', '--temp', metavar='dir', help='A directory to temporarily extract files into', type=Path, default=None)
     parser.add_argument('-e', '--embed', help='Create an embedding layout', action='store_true', default=False)
-    parser.add_argument('-b', '--build', help='Specify the build directory', type=Path)
+    parser.add_argument('-b', '--build', help='Specify the build directory', type=Path, default=None)
     ns = parser.parse_args()
 
     source = ns.source or (Path(__file__).resolve().parent.parent.parent)
     out = ns.out
-    build = ns.build
+    build = ns.build or Path(sys.exec_prefix)
     assert isinstance(source, Path)
     assert not out or isinstance(out, Path)
     assert isinstance(build, Path)
@@ -226,8 +230,13 @@ def main():
             copied = copy_to_layout(temp / t.rstrip('/'), chain(files, extra_files))
             print('Copied {} files'.format(copied))
 
-        with open(str(temp / 'pyvenv.cfg'), 'w') as f:
-            print('applocal = true', file=f)
+        if ns.embed:
+            with open(str(temp / (BASE_NAME + '._pth')), 'w') as f:
+                print(BASE_NAME + '.zip', file=f)
+                print('.', file=f)
+                print('', file=f)
+                print('# Uncomment to run site.main() automatically', file=f)
+                print('#import site', file=f)
 
         if out:
             total = copy_to_layout(out, rglob(temp, '**/*', None))
